/* 
   Copyright (C) 2010, 2011 German A. Arias <german@xelalug.org>

   This file is part of Gemas application

   Gemas is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
 
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSTask.h>
#import "GemasController.h"
#import "GemasDocument.h"

#define POST_CHANGE \
  [[NSNotificationCenter defaultCenter] \
    postNotificationName: CodeEditorDefaultsDidChangeNotification \
                  object: self \
                userInfo: nil]


@implementation GemasController

- (void) awakeFromNib
{
  //GSmarkupBrowser = nil;
}

- (void) newSourceFile: (id)sender
{
  switch([sender tag])
    {
    case 1:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"C" display: YES];
      break;
    case 2:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"C-Header" display: YES];
      break;
    case 3:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"ObjC" display: YES];
      break;
    case 4:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"C++" display: YES];
      break;
    case 5:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"ObjC++" display: YES];
      break;
    case 6:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"Plist" display: YES];
      break;
    case 7:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"Strings" display: YES];
      break;
    case 8:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"GSmarkup" display: YES];
      break;
    case 9:
      [[NSDocumentController sharedDocumentController]
        openUntitledDocumentOfType: @"GNUmakefile" display: YES];
      break;
    }
}

- (void) openPreferences: (id)sender
{
  if (preferencesPanel == nil)
    {
      [NSBundle loadNibNamed: @"Preferences" owner: self];
      [preferencesPanel center];
      [preferencesPanel makeKeyAndOrderFront: self];
      
      NSUserDefaults * df = [NSUserDefaults standardUserDefaults];

      //Editing
      [indentation selectItemAtIndex: [df integerForKey: @"Indentation"]];

      [tabConversion selectItemAtIndex: [df integerForKey: @"TabConversion"]];     
      
      //Looks
      NSData *data = [df dataForKey: @"EditorTextColor"];
      if (data != nil)
        {
          [textColor setColor: [NSKeyedUnarchiver unarchiveObjectWithData: data]];
        }
      
      data = [df dataForKey: @"EditorBackgroundColor"];
      if (data != nil)
        {
          [backgroundColor setColor: [NSKeyedUnarchiver unarchiveObjectWithData:
                                                          data]];
        }

      data = [df dataForKey: @"EditorInsertionPointColor"];
      if (data != nil)
        {
          [cursorColor setColor: [NSKeyedUnarchiver
                                   unarchiveObjectWithData: data]];
        }
    }
  else
    {
      [preferencesPanel makeKeyAndOrderFront: self];
    }
}

- (void) openGoToLinePanel: (id)sender
{
  [NSBundle loadNibNamed: @"GoToLine" owner: self];
  [NSApp runModalForWindow:linePanel];
}

- (void) goToLine: (id) sender
{
  [[[NSDocumentController sharedDocumentController] currentDocument]
    goToLineNumber: [sender intValue]];
  [[sender window] performClose: self];    
}


//Editing

- (void) changeIndentation: (id)sender
{
  [[NSUserDefaults standardUserDefaults]
    setInteger: [indentation indexOfSelectedItem]
        forKey: @"Indentation"];

  POST_CHANGE;
}

- (void) changeTabConversion: (id)sender
{
  [[NSUserDefaults standardUserDefaults]
    setInteger: [tabConversion indexOfSelectedItem]
        forKey: @"TabConversion"];
}


//Looks

- (void) changeCursorColor: (id)sender
{
  [[NSUserDefaults standardUserDefaults]
    setObject: [NSKeyedArchiver archivedDataWithRootObject: [cursorColor color]]
       forKey: @"EditorInsertionPointColor"];

  POST_CHANGE;
}

- (void) changeTextColor: (id)sender
{
  [[NSUserDefaults standardUserDefaults]
    setObject: [NSKeyedArchiver archivedDataWithRootObject: [textColor color]]
       forKey: @"EditorTextColor"];

  POST_CHANGE;
}

- (void) changeBackgroundColor: (id)sender
{
  [[NSUserDefaults standardUserDefaults]
    setObject: [NSKeyedArchiver archivedDataWithRootObject: [backgroundColor color]]
       forKey: @"EditorBackgroundColor"];

  POST_CHANGE;
}

//Tools
/*- (void) testGSmarkup: (id)sender
{
  NSString *autoLayout, *pathMB;
  NSString *filePath = [[[NSDocumentController sharedDocumentController] currentDocument] fileName];

  if (GSmarkupBrowser != nil)
    {
      NSLog(@"Se termina");
      [GSmarkupBrowser terminate];
    }
  
  if ([sender tag] == 1)
    {
      autoLayout = [NSString stringWithString: @"NO"];
    }
  else
    {
      autoLayout = [NSString stringWithString: @"YES"];
    }
  
  pathMB = [[NSWorkspace sharedWorkspace] fullPathForApplication: @"GSMarkupBrowser"];
  pathMB = [pathMB stringByAppendingString: @"/GSMarkupBrowser"];
  
  GSmarkupBrowser = [NSTask launchedTaskWithLaunchPath: pathMB arguments: [NSArray arrayWithObjects: filePath, @"-DisplayAutoLayout", autoLayout, nil]];

}

- (void) terminateGSmarkupTest: (id)sender
{
  if (GSmarkupBrowser != nil)
    {
      [GSmarkupBrowser terminate];
    }
    }*/

//Delegate
- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
  if ([[menuItem title] isEqualToString: _(@"Go To Line...")])
    {
      if ([[[NSDocumentController sharedDocumentController] documents] count] > 0)
        {
          return YES;
        }
      else
        {
          return NO;
        }
    }
 
  return [[NSDocumentController sharedDocumentController] validateMenuItem: menuItem];
}

- (BOOL) applicationShouldOpenUntitledFile: (NSApplication *) sender
{
  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) ==
      NSWindows95InterfaceStyle)
    {
      return YES;
    }
  else
    {
      return NO;
    }
}

- (void) windowWillClose: (NSNotification*)aNotification
{
  [NSApp stopModal];
}

@end
