/* 
   Copyright (C) 2010, 2011 German A. Arias <german@xelalug.org>

   This file is part of Gemas application

   Gemas is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
 
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "GemasEditorView.h"
#import <Foundation/Foundation.h>
#import "GemasDocument.h"
#import <HighlighterKit/HighlighterKit.h>

@interface GemasEditorView (Private)

- (BOOL) isGSmarkupIndent: (NSString *)string;
- (BOOL) isGSmarkupBackIndent: (NSString *)string;

@end

@implementation GemasEditorView (Private)

- (BOOL) isGSmarkupIndent: (NSString *)string
{
  if ([type isEqualToString: @"GSmarkup"])
    {
      if ([string hasPrefix: @"<"] &&
	  [string hasSuffix: @">"] &&
	  ![string hasPrefix: @"<!"] &&
	  ![string hasPrefix: @"<?"] &&
	  ![string hasPrefix: @"</"] &&
	  ![string hasSuffix: @"/>"] &&
	  ![string hasSuffix: @"</label>"] &&
	  ![string hasSuffix: @"</textField>"] &&
	  ![string hasSuffix: @"</secureTextField>"])
        {
          return YES;
        }
      else
        {
          return NO;
        }
    }
  else
    {
      return NO;
    }
}

- (BOOL) isGSmarkupBackIndent: (NSString *)string
{
  return NO;
}

@end

@implementation GemasEditorView

- (void) dealloc
{
  TEST_RELEASE(highlighter);
  TEST_RELEASE(openCharacters);
  TEST_RELEASE(closeCharacters);
  TEST_RELEASE(indentCharacters);
  TEST_RELEASE(backIndentCharacters);
  TEST_RELEASE(type);
  [super dealloc];
}

- (void) awakeFromNib
{
  openCharacters = [NSCharacterSet characterSetWithCharactersInString: @"[{("];
  [openCharacters retain];
  closeCharacters = [NSCharacterSet characterSetWithCharactersInString: @"]})"];
  [closeCharacters retain];
  indentCharacters = [NSCharacterSet characterSetWithCharactersInString: @"{"];
  [indentCharacters retain];
  backIndentCharacters = [NSCharacterSet characterSetWithCharactersInString: @"}"];
  [backIndentCharacters retain];
}

- (void) drawRect: (NSRect) frame
{
  NSLayoutManager *layoutManager;
  NSTextContainer *textContainer;
  NSRange drawnRange;
  
  layoutManager = [self layoutManager];
  textContainer = [self textContainer];
  drawnRange = [layoutManager glyphRangeForBoundingRect: frame
                              inTextContainer: textContainer];
  
  [highlighter highlightRange: drawnRange];
  [super drawRect: frame];
}


//Insert the correponding indentation when the user press Tab key
- (void) insertTab: (id)sender
{
  switch ([[NSUserDefaults standardUserDefaults]
            integerForKey: @"Indentation"])
    {
    case 0:  // 2 spaces
      [super insertText: @"  "];
      break;
    case 1:  // 4 spaces
      [super insertText: @"    "];
      break;
    }
}

//If the new line need indentation, add it
- (void) insertNewline: (id)sender
{
  NSString *spaceToInsert, *previousLine = nil;
  
  [super insertNewline: sender];
  NSRange textRange = [self selectedRange];
  NSArray *lines = [[[self string] substringFromRange: NSMakeRange(0, textRange.location)]
                     componentsSeparatedByString: @"\n"];

  if ([lines count] >= 3)
    {
      previousLine = [lines objectAtIndex: [lines count] - 3];
      previousLine = [previousLine stringByReplacingOccurrencesOfString:
				     @" " withString: @""];
    }

  NSString *lastLine = [lines objectAtIndex: [lines count] - 2];
  
  if ([[NSCharacterSet whitespaceAndNewlineCharacterSet] isSupersetOfSet: [NSCharacterSet characterSetWithCharactersInString: lastLine]])
    {
      spaceToInsert = [NSString stringWithString: lastLine];
    }
  else
    {
      NSInteger tab = [[NSUserDefaults standardUserDefaults]
                        integerForKey: @"Indentation"];
      NSString *checkLine = [lastLine stringByReplacingOccurrencesOfString: @" " withString: @""];
      spaceToInsert = [lastLine stringByDeletingSuffix: [lastLine stringByTrimmingLeadSpaces]];
      
      unichar lastChar = [[lastLine stringByTrimmingTailSpaces] characterAtIndex: [[lastLine stringByTrimmingTailSpaces] length] - 1];
      
      if ((([indentCharacters characterIsMember: lastChar]) &&
	   (![previousLine hasPrefix: @"switch("])) ||
	  ([checkLine hasPrefix: @"if("]) ||
	  ([checkLine hasPrefix: @"else"]) ||
	  ([checkLine hasPrefix: @"while("] && ![previousLine hasPrefix: @"}"]) ||
	  ([checkLine hasPrefix: @"for("]) ||
	  ([checkLine isEqualToString: @"do"]) ||
	  ([checkLine isEqualToString: @"default:"]) ||
	  ([checkLine hasPrefix: @"switch("]) ||
	  ([checkLine hasPrefix: @"case"]) ||
	  ([self isGSmarkupIndent: checkLine]))
        {
          if (tab == 0)
            {
              spaceToInsert = [spaceToInsert stringByAppendingString: @"  "];
            }
          
          if (tab == 1)
            {
              spaceToInsert = [spaceToInsert stringByAppendingString: @"    "];
            }
        }
      else
        {
          if (([backIndentCharacters characterIsMember: lastChar]) ||
	      ([checkLine hasSuffix: @"break;"]) ||
	      ([self isGSmarkupBackIndent: checkLine]))
            {
              if ((tab == 0) && ([spaceToInsert length] >= 2))
                {
                  spaceToInsert = [spaceToInsert stringByDeletingPrefix: @"  "];
                }
              
              if ((tab == 1) && ([spaceToInsert length] >= 4))
                {
                  spaceToInsert = [spaceToInsert stringByDeletingPrefix: @"    "];
                }
            }
        }
    }

  [super insertText: spaceToInsert];
}

- (void) insertText: (id)string
{
  if ([string isKindOfClass: [NSString class]])
    {
      if (([backIndentCharacters characterIsMember: [string characterAtIndex: 0]]) || ([type isEqualToString: @"GSmarkup"] && [string isEqualToString: @"/"]))
        {
          BOOL gsmarkup = NO;
          NSRange textRange = [self selectedRange];
          NSArray *lines = [[[self string] substringFromRange: NSMakeRange(0, textRange.location)]
                             componentsSeparatedByString: @"\n"];
          NSString *lastLine = [lines objectAtIndex: [lines count] - 1];
          
          int length;
          NSString *stringToReplace;
          NSCharacterSet *spaces = [NSCharacterSet whitespaceCharacterSet];
          NSCharacterSet *charsLine = [NSCharacterSet characterSetWithCharactersInString: lastLine];
          NSInteger tab = [[NSUserDefaults standardUserDefaults]
                            integerForKey: @"Indentation"];

          //Check if is a gsmarkup character
          if ([[lastLine stringByTrimmingLeadSpaces] isEqualToString: @"<"] && [type isEqualToString: @"GSmarkup"])
            {
              gsmarkup = YES;
            }
          
          if ((([lastLine length] > 0) && ([spaces isSupersetOfSet: charsLine])) || gsmarkup)
            {
              stringToReplace = [NSString stringWithString: lastLine];
            }
          else
            {
              stringToReplace = [NSString stringWithString: @""];
            }
          
          length = [stringToReplace length];
          
          if (!gsmarkup && (tab == 0) && ([stringToReplace length] >= 2))
            {
              stringToReplace = [stringToReplace stringByDeletingPrefix: @"  "];
            }
          
          if (!gsmarkup && (tab == 1) && ([stringToReplace length] >= 4))
            {
              stringToReplace = [stringToReplace stringByDeletingPrefix: @"    "];
            }

          //GSmarkup backIndent
          if (gsmarkup && (tab == 0) && ([stringToReplace length] >= 3))
            {
              stringToReplace = [stringToReplace stringByDeletingPrefix: @"  "];
            }
          
          if (gsmarkup && (tab == 1) && ([stringToReplace length] >= 5))
            {
              stringToReplace = [stringToReplace stringByDeletingPrefix: @"    "];
            }
          
          [self replaceCharactersInRange: NSMakeRange(textRange.location - length, length) withString: stringToReplace];
          [super insertText: string];
        }
      else
        {
          [super insertText: string];
        }
    }
  else
    {
      [super insertText: string];
    }
}

/*Select the range of parentheses when the user do a double 
  click obove one*/
- (void) mouseDown: (NSEvent *)event 
{
  [super mouseDown: event];
  NSRange textRange = [self selectedRange];
  NSString *selection = [[self string] substringFromRange: textRange];

  if ([selection length] == 1)
    {
      //Open character {[(
      if ([openCharacters characterIsMember: [selection characterAtIndex: 0]])
        {
          NSString *search = nil;
          NSRange firstClose, firstOpen;
          NSString *contextA, *contextB;
          NSString *context = [[self string] substringFromRange: NSMakeRange(textRange.location + 1, [[self string] length] - textRange.location - 1)];

          if ([selection isEqualToString: @"{"])
            {
              search = [NSString stringWithString: @"}"];
            }

          if ([selection isEqualToString: @"["])
            {
              search = [NSString stringWithString: @"]"];
            }

          if ([selection isEqualToString: @"("])
            {
              search = [NSString stringWithString: @")"];
            }

          firstOpen = [context rangeOfString: selection];
          firstClose = [context rangeOfString: search];

          firstOpen = NSMakeRange(firstOpen.location + 1, firstOpen.length);
          firstClose = NSMakeRange(firstClose.location + 1, firstClose.length);

          while ((firstOpen.location < firstClose.location) && (firstOpen.length != 0) && (firstClose.length != 0))
            {
              contextA = [[self string] substringFromRange: NSMakeRange(textRange.location + firstOpen.location + 1, [[self string] length] - firstOpen.location - textRange.location - 1)];
              contextB = [[self string] substringFromRange: NSMakeRange(textRange.location + firstClose.location + 1, [[self string] length] - firstClose.location - textRange.location - 1)];
              
              firstOpen = NSMakeRange([[self string] length] - [contextA length] + [contextA rangeOfString: selection].location - textRange.location, [contextA rangeOfString: selection].length);
              firstClose = NSMakeRange([[self string] length] - [contextB length] + [contextB rangeOfString: search].location - textRange.location, [contextB rangeOfString: search].length);
            }

          [self setSelectedRange: NSMakeRange(textRange.location, firstClose.location + 1)];
        }
      
      //Close character }])
      if ([closeCharacters characterIsMember: [selection characterAtIndex: 0]])
        {
          NSString *search = nil;
          NSRange firstClose, firstOpen;
          NSString *contextA, *contextB;
          NSString *context = [[self string] substringFromRange: NSMakeRange(0, textRange.location)];

          if ([selection isEqualToString: @"}"])
            {
              search = [NSString stringWithString: @"{"];
            }

          if ([selection isEqualToString: @"]"])
            {
              search = [NSString stringWithString: @"["];
            }

          if ([selection isEqualToString: @")"])
            {
              search = [NSString stringWithString: @"("];
            }

          firstOpen = [context rangeOfString: search options: NSBackwardsSearch];
          firstClose = [context rangeOfString: selection options: NSBackwardsSearch];

          while ((firstOpen.location < firstClose.location) && (firstClose.length != 0))
            {
              contextA = [[self string] substringFromRange: NSMakeRange(0, firstOpen.location)];
              contextB = [[self string] substringFromRange: NSMakeRange(0, firstClose.location)];
              
              firstOpen = [contextA rangeOfString: search options: NSBackwardsSearch];
              firstClose = [contextB rangeOfString: selection options: NSBackwardsSearch];
              
            }
          
          [self setSelectedRange: NSMakeRange(firstOpen.location, textRange.location - firstOpen.location + 1)];
        }
    }
}

//Creates a new syntax highlighter for the specified file type
- (void) createSyntaxHighlighterForFileType: (NSString *) fileType
{
  type = [NSString stringWithString: fileType];
  [type retain];

  ASSIGN (highlighter, [[[HKSyntaxHighlighter alloc]
                          initWithHighlighterType: fileType
                          textStorage: [self textStorage]
                          defaultTextColor: [editorDocument textColor]]
                         autorelease]);

  if ([type isEqualToString: @"C"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.c"]];
    }

  if ([type isEqualToString: @"C-Header"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.h"]];
    }

  if ([type isEqualToString: @"C++"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.cc"]];
    }

  if ([type isEqualToString: @"ObjC"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.m"]];
    }

  if ([type isEqualToString: @"ObjC++"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.mm"]];
    }

  if ([type isEqualToString: @"Plist"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.plist"]];
    }

  if ([type isEqualToString: @"Strings"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.strings"]];
    }

  if ([type isEqualToString: @"GSmarkup"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_.gsmarkup"]];
    }

  if ([type isEqualToString: @"GNUmakefile"])
    {
      [[self window] setMiniwindowImage: [NSImage imageNamed: @"FileIcon_makefile"]];
    }
}

@end
